/**
 * @file unit_tests.c
 *
 * @brief Examples for implementation of unit tests
 * @warning This software utilizes TI Drivers
 *
 * \copyright Copyright (C) 2022 Texas Instruments Incorporated - http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "unit_tests.h"


// Number of samples to collect for noise testing
#define NUM_SAMPLES_NOISE   (20)

// Helper functions
static void     readSingleChannelPPNoise(int32_t *dataArray, uint8_t channelNum);


/* Unit-tests
 *
 *
 * NOTES:
 * - All communication is based on TI Drivers implementation.
 * - Unless specified, tests should be self-contained (results should not depend on the testing sequence)!
 * - Tests that modify register configurations should restore the device to its default state.
 * - All test should return a boolean data type!
 * - All test conditions should be a constant data type and defined within the specific test function.
 * - All test functions and conditions should be documented to explain their significance.
 *
 * - In case the device is not working, avoid polling nDRDY to avoid the possibility of software entering into an infinite loop!
 * - Avoid writing tests that verify your code's functionality. Check code functionality elsewhere (through unit/functional tests)!
 */


/************************************************************************************//**
 *
 * @brief run_self_tests()
 *             Run all test cases
 *
 * @return      true for successful tests
 *              false for unsuccessful tests
 *
 */
bool run_self_tests(void)
{
    bool allTestsPassed = true;

    delay_ms(100);   // Delay in case we are calling this function while supply rails are still settling.

    // GPIO Functionality

    // NOTE: All tests assume GPIO pins are controlled by MCU.
    // If pins are tied, update the tests to check for desired state.

    allTestsPassed &= test_RESET_pin();
	allTestsPassed &= test_START_SYNC_pin();
    allTestsPassed &= test_CS_pin();
    allTestsPassed &= test_DRDY_interrupt();


    // SPI Functionality

    allTestsPassed &= test_read_register();
    allTestsPassed &= test_write_register();
    allTestsPassed &= test_reset_command();


    // ADC Performance

    allTestsPassed &= test_noise();

    // CRC Test

    allTestsPassed &= test_CRC();

    return allTestsPassed;
}


/*   SPECIFIC TEST CASES    */

/* GPIO Functionality */
/************************************************************************************//**
 *
 * @brief test_RESET_pin()
 *             Performs test of the RESET pin
 *
 *
 * @return      true for successful tests
 *              false for unsuccessful tests
 *
 */
bool test_RESET_pin(void)
{
    bool b_pass = true;
    bool setting;

    // Set nRESET LOW and read it back
    setRESET(LOW);
    setting = getRESET();
    b_pass &= (setting == LOW);

    // Set nRESET HIGH and read it back
    setRESET(HIGH);
    setting = getRESET();
    b_pass = (setting == HIGH);

    // Make sure there is delay following reset for internal ADC processes to restart
    delay_us( DELAY_4096TCLK );

    // Ensure internal register array is initialized
    restoreRegisterDefaults();

    return b_pass;
}

/************************************************************************************//**
 *
 * @brief test_START_SYNC_pin()
 *             Performs test of the START/SYNC pin
 *
 * @return      true for successful tests
 *              false for unsuccessful tests
 *
 */
bool test_START_SYNC_pin(void)
{
    bool b_pass = true;
    bool setting;

    // Set START/SYNC LOW and read it back
    setSTART(LOW);
    setting = getSTART();
    b_pass &= (setting == LOW);

    // Set START/SYNC HIGH and read it back
    setSTART(HIGH);
    setting = getSTART();
    b_pass = (setting == HIGH);

    return b_pass;
}

/************************************************************************************//**
 *
 * @brief test_CS_pin()
 *             Performs test of the CS pin
 *
 * @return      true for successful tests
 *              false for unsuccessful tests
 *
 */
bool test_CS_pin(void)
{
    bool b_pass = true;
    bool setting;

    // Set nCS LOW and read it back
    setCS(LOW);
    setting = getCS();
    b_pass &= (setting == LOW);

    // Set nCS HIGH and read it back
    setCS(HIGH);
    setting = getCS();
    b_pass = (setting == HIGH);

    return b_pass;
}

/************************************************************************************//**
 *
 * @brief test_DRDY_interrupt()
 *             Performs test of the DRDY pin
 *
 * @return      true for successful tests
 *              false for unsuccessful tests
 *
 */
bool test_DRDY_interrupt(void)
{
    bool b_pass = true;

    enableDRDYinterrupt(true);

    // Toggle START/SYNC pin high and check if nDRDY is active and interrupt is functioning
    toggleSTART(false);  // Abort current conversion

    // Check to see if a DRDY pin interrupt has occurred
    b_pass &= waitForDRDYHtoL(100);

    enableDRDYinterrupt(false);

    return b_pass;
}

/* SPI Functionality */
/************************************************************************************//**
 *
 * @brief test_read_register()
 *             Performs test of the communication for reading registers
 *
 * @return      true for successful tests
 *              false for unsuccessful tests
 *
 */
bool test_read_register(void)
{
    bool b_pass = true;
    uint8_t value;

    toggleRESET();
    // Make sure there is delay following reset for internal ADC processes to restart
    delay_us( DELAY_4096TCLK );

    // Ensure internal register array is initialized
    restoreRegisterDefaults();

    // Check the value of the STATUS register
    value = readSingleRegister(spiHdl, REG_ADDR_STATUS);
    b_pass &= (value == STATUS_DEFAULT);

    return b_pass;
}

/************************************************************************************//**
 *
 * @brief test_write_register()
 *             Performs test of the communications for writing registers
 *
 * @return      true for successful tests
 *              false for unsuccessful tests
 *
 */
bool test_write_register(void)
{
    bool b_pass = true;
    uint8_t writeValue, readValue;

    writeValue = ADS_PGA_ENABLED | ADS_GAIN_128;
    writeSingleRegister(spiHdl, REG_ADDR_PGA, writeValue);

    readValue = readSingleRegister(spiHdl, REG_ADDR_PGA);
    b_pass &= (readValue == writeValue);

    return b_pass;
}

/************************************************************************************//**
 *
 * @brief test_reset_command()
 *             Performs test of the RESET command
 *
 * @return      true for successful tests
 *              false for unsuccessful tests
 *
 */
bool test_reset_command(void)
{
    bool b_pass = true;
    uint16_t writeValue, readValue;

    // Write non-default value to register
    writeValue = ADS_PGA_ENABLED | ADS_GAIN_128;
    writeSingleRegister(spiHdl, REG_ADDR_PGA, writeValue);

    // Issue reset command
    sendRESET(spiHdl);

    // Make sure there is delay following reset for internal ADC processes to restart
    delay_us( DELAY_4096TCLK );

    // Ensure internal register array is initialized
    restoreRegisterDefaults();

    // Check if read register command returns default value
    readValue = readSingleRegister(spiHdl, REG_ADDR_PGA);
    b_pass &= (readValue == PGA_DEFAULT);

    return b_pass;
}

/* ADC Performance  */
/************************************************************************************//**
 *
 * @brief test_noise()
 *             Performs test of the ADC functionality and performance
 *
 * @return      true for successful tests
 *              false for unsuccessful tests
 *
 */
bool test_noise(void)
{
    bool b_pass = true;
    int i;
    uint8_t writeValue;
    int32_t maxValue, minValue;
    int32_t channelData[NUM_SAMPLES_NOISE];

    const int32_t pp_limit = 220;    // 0.51 uVpp

    // Ensure device is powered, converting, and reset to default values
     toggleRESET();

    // Make sure there is delay following reset for internal ADC processes to restart
    delay_us( DELAY_4096TCLK );

    // Ensure internal register array is initialized
    restoreRegisterDefaults();

    // Test configuration
    // Configure device to use PGA and gain of 128
    writeValue = ADS_PGA_ENABLED | ADS_GAIN_128;
    writeSingleRegister(spiHdl, REG_ADDR_PGA, writeValue);

    // Configure device to use internal reference and turn reference always on
    writeValue = ADS_REFSEL_INT | ADS_REFINT_ON_ALWAYS;
    writeSingleRegister(spiHdl, REG_ADDR_REF, writeValue);

    // Configure device to use shorted inputs at (AVDD-AVSS)/2 is SYS register
    writeValue = ADS_SYS_MON_SHORT;
    writeSingleRegister(spiHdl, REG_ADDR_SYS, writeValue);

    // Send a START command to start taking conversions
    enableDRDYinterrupt(true);
    sendSTART( spiHdl );

    // Collect data
    readSingleChannelPPNoise(channelData, 1);

    enableDRDYinterrupt(false);
    sendSTOP( spiHdl );

    // Calculate PP Noise
    minValue = channelData[0];
    maxValue = channelData[0];
    for(i = 1; i < NUM_SAMPLES_NOISE; i++)
    {
        if (channelData[i] < minValue)      { minValue = channelData[i]; }
        else if (channelData[i] > maxValue) { maxValue = channelData[i]; }
    }

    // Check PP Noise performance
    b_pass &= (maxValue - minValue) <= pp_limit;

    return b_pass;
}

/************************************************************************************//**
 *
 * @brief test_CRC()
 *             Performs test of the CRC functionality
 *
 * @return      true for successful tests
 *              false for unsuccessful tests
 *
 */
bool test_CRC(void)
{
    bool b_pass = true;
    int i;
    uint8_t writeValue;
    int32_t maxValue, minValue;
    int32_t channelData[NUM_SAMPLES_NOISE];

    const int32_t pp_limit = 220;    // 0.51 uVpp

    // Test configuration
    // Configure device to use PGA and gain of 128
    writeValue = ADS_PGA_ENABLED | ADS_GAIN_128;
    writeSingleRegister(spiHdl, REG_ADDR_PGA, writeValue);

    // Configure device to use internal reference and turn reference always on
    writeValue = ADS_REFSEL_INT | ADS_REFINT_ON_ALWAYS;
    writeSingleRegister(spiHdl, REG_ADDR_REF, writeValue);

    // Configure device to use shorted inputs at (AVDD-AVSS)/2 is SYS register
    writeValue = ADS_SYS_MON_SHORT | ADS_CRC_ENABLE | ADS_SENDSTATUS_ENABLE;
    writeSingleRegister(spiHdl, REG_ADDR_SYS, writeValue);

    // Send a START command to start taking conversions
    enableDRDYinterrupt(true);
    sendSTART( spiHdl );

    // Collect data
    readSingleChannelPPNoise(channelData, 1);

    enableDRDYinterrupt(false);
    sendSTOP( spiHdl );

    // Calculate PP Noise
    minValue = channelData[0];
    maxValue = channelData[0];
    for(i = 1; i < NUM_SAMPLES_NOISE; i++)
    {
        if (channelData[i] < minValue)      { minValue = channelData[i]; }
        else if (channelData[i] > maxValue) { maxValue = channelData[i]; }
    }

    // Check PP Noise performance
    b_pass &= (maxValue - minValue) <= pp_limit;

    return b_pass;
}

/************************************************************************************//**
 *
 * @brief readSingleChannelPPNoise()
 *             Reads channel data for noise analysis
 *
 * @param[in]   *dataArray    pointer to the data array
 * @param[in]   channelNum    channel number to be converted
 *
 * @return      None
 *
 */
static void readSingleChannelPPNoise(int32_t *dataArray, uint8_t channelNum)
{
    int i;
    uint8_t status[] = {0};
    readMode mode = COMMAND;

    // Read data
    for(i = 0; i < NUM_SAMPLES_NOISE; i++)
    {
        waitForDRDYHtoL(100);
        dataArray[i] = readConvertedData(spiHdl, status, mode);
    }
}
